Package: patchwork
Type: Package
Title: The Composer of Plots
Version: 1.3.2
Authors@R: 
    person(given = "Thomas Lin",
           family = "Pedersen",
           role = c("cre", "aut"),
           email = "thomasp85@gmail.com",
           comment = c(ORCID = "0000-0002-5147-4711"))
Maintainer: Thomas Lin Pedersen <thomasp85@gmail.com>
Description: The 'ggplot2' package provides a strong API for sequentially 
    building up a plot, but does not concern itself with composition of multiple
    plots. 'patchwork' is a package that expands the API to allow for 
    arbitrarily complex composition of plots by, among others, providing 
    mathematical operators for combining multiple plots. Other packages that try 
    to address this need (but with a different approach) are 'gridExtra' and 
    'cowplot'.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: ggplot2 (>= 3.0.0), gtable (>= 0.3.6), grid, stats, grDevices,
        utils, graphics, rlang (>= 1.0.0), cli, farver
RoxygenNote: 7.3.2
URL: https://patchwork.data-imaginist.com,
        https://github.com/thomasp85/patchwork
BugReports: https://github.com/thomasp85/patchwork/issues
Suggests: knitr, rmarkdown, gridGraphics, gridExtra, ragg, testthat (>=
        2.1.0), vdiffr, covr, png, gt (>= 0.11.0)
VignetteBuilder: knitr
Config/Needs/website: gifski
NeedsCompilation: no
Packaged: 2025-08-25 08:14:43 UTC; thomas
Author: Thomas Lin Pedersen [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-5147-4711>)
Repository: CRAN
Date/Publication: 2025-08-25 10:00:02 UTC
Built: R 4.6.0; ; 2026-04-27 21:31:25 UTC; unix
