# Generated by roxygen2: do not edit by hand

S3method("&","ggplot2::gg")
S3method("&",gg)
S3method("*","ggplot2::gg")
S3method("*",gg)
S3method("-",ggplot)
S3method("/",ggplot)
S3method("[[",patchwork)
S3method("[[<-",patchwork)
S3method("|",ggplot)
S3method(add_patches,ggplot)
S3method(add_patches,patchwork)
S3method(as.gtable,patchwork)
S3method(as.list,patchwork)
S3method(as_patch,formula)
S3method(as_patch,gList)
S3method(as_patch,ggplot)
S3method(as_patch,grob)
S3method(as_patch,gt_tbl)
S3method(as_patch,nativeRaster)
S3method(as_patch,patchwork)
S3method(as_patch,raster)
S3method(as_patchwork,default)
S3method(as_patchwork,ggplot)
S3method(as_patchwork,patchwork)
S3method(c,patch_area)
S3method(get_dim,ggplot)
S3method(get_dim,patchwork)
S3method(ggplot_add,formula)
S3method(ggplot_add,ggplot)
S3method(ggplot_add,grob)
S3method(ggplot_add,gt_tbl)
S3method(ggplot_add,nativeRaster)
S3method(ggplot_add,plot_annotation)
S3method(ggplot_add,plot_layout)
S3method(ggplot_add,raster)
S3method(ggplot_build,fixed_dim_ggplot)
S3method(ggplot_gtable,fixed_dim_build)
S3method(has_tag,ggplot)
S3method(has_tag,guide_area)
S3method(has_tag,inset_patch)
S3method(has_tag,plot_filler)
S3method(has_tag,spacer)
S3method(has_tag,wrapped_patch)
S3method(length,patch_area)
S3method(length,patchwork)
S3method(merge,ggplot)
S3method(merge,patchwork)
S3method(names,patchwork)
S3method(patchGrob,guide_area)
S3method(patchGrob,patch)
S3method(patchGrob,table_patch)
S3method(patchGrob,wrapped_patch)
S3method(patchGrob,wrapped_table)
S3method(plot,inset_patch)
S3method(plot,patch)
S3method(plot,patch_area)
S3method(plot,patchwork)
S3method(plot_table,free_plot)
S3method(plot_table,ggplot)
S3method(plot_table,inset_patch)
S3method(plot_table,patch)
S3method(plot_table,patchwork)
S3method(print,inset_patch)
S3method(print,patch)
S3method(print,patch_area)
S3method(print,patchwork)
S3method(print,plot_dimension)
S3method(set_dim,ggplot)
S3method(set_dim,patchwork)
S3method(simplify_gt,free_table)
S3method(simplify_gt,gtable)
S3method(simplify_gt,gtable_patchwork)
S3method(simplify_gt,inset_table)
S3method(simplify_gt,patchgrob)
S3method(str,patchwork)
export(align_patches)
export(align_plots)
export(area)
export(free)
export(get_dim)
export(get_max_dim)
export(guide_area)
export(inset_element)
export(patchGrob)
export(patchworkGrob)
export(plot_annotation)
export(plot_layout)
export(plot_spacer)
export(set_dim)
export(wrap_elements)
export(wrap_ggplot_grob)
export(wrap_plots)
export(wrap_table)
import(cli)
import(rlang)
importFrom(farver,get_channel)
importFrom(farver,set_channel)
importFrom(ggplot2,aes)
importFrom(ggplot2,calc_element)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_grob)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_render)
importFrom(ggplot2,element_text)
importFrom(ggplot2,find_panel)
importFrom(ggplot2,geom_blank)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplotGrob)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,is_ggplot)
importFrom(ggplot2,is_theme)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,max_height)
importFrom(ggplot2,max_width)
importFrom(ggplot2,panel_cols)
importFrom(ggplot2,panel_rows)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_reverse)
importFrom(ggplot2,set_last_plot)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_get)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,waiver)
importFrom(ggplot2,wrap_dims)
importFrom(ggplot2,zeroGrob)
importFrom(grDevices,is.raster)
importFrom(grid,absolute.size)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,editGrob)
importFrom(grid,gTree)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(grid,grobHeight)
importFrom(grid,grobTree)
importFrom(grid,grobWidth)
importFrom(grid,heightDetails)
importFrom(grid,is.grob)
importFrom(grid,is.unit)
importFrom(grid,pushViewport)
importFrom(grid,rasterGrob)
importFrom(grid,seekViewport)
importFrom(grid,unit)
importFrom(grid,unit.c)
importFrom(grid,unit.pmax)
importFrom(grid,upViewport)
importFrom(grid,valid.just)
importFrom(grid,viewport)
importFrom(grid,widthDetails)
importFrom(gtable,as.gtable)
importFrom(gtable,gtable)
importFrom(gtable,gtable_add_cols)
importFrom(gtable,gtable_add_grob)
importFrom(gtable,gtable_add_rows)
importFrom(gtable,gtable_height)
importFrom(gtable,gtable_width)
importFrom(gtable,is.gtable)
importFrom(stats,ave)
importFrom(stats,na.omit)
importFrom(utils,as.roman)
importFrom(utils,getFromNamespace)
importFrom(utils,modifyList)
importFrom(utils,str)
importFrom(utils,tail)
