% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_multipage.R
\name{align_plots}
\alias{align_plots}
\title{Deprecated functions}
\description{
These functions are deprecated and should not be used.
}
\keyword{internal}
